@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfb.c 8611 2008-03-27 08:37:58Z ken $ */
+/* $Id: gdevpdfb.c 11971 2010-12-22 16:17:58Z ken $ */
 /* Low-level bitmap image handling for PDF-writing driver */
 #include "string_.h"
 #include "gx.h"
@@ -23,6 +23,10 @@
 #include "gxdcolor.h"
 #include "gxpcolor.h"
 #include "gxhldevc.h"
+#include "gxchar.h"
+#include "gdevpdtf.h"		/* Required to include gdevpdti.h */
+#include "gdevpdti.h"		/* For pdf_charproc_x_offset */
+#include "gsptype1.h"
 
 /* We need this color space type for constructing temporary color spaces. */
 extern const gs_color_space_type gs_color_space_type_Indexed;
@@ -115,7 +119,6 @@ set_image_color(gx_device_pdf *pdev, gx_color_index c)
 			    &psdf_set_stroke_color_commands);
 }
 
-/* Copy a monochrome bitmap or mask. */
 static int
 pdf_copy_mono(gx_device_pdf *pdev,
 	      const byte *base, int sourcex, int raster, gx_bitmap_id id,
@@ -133,6 +136,9 @@ pdf_copy_mono(gx_device_pdf *pdev,
     pdf_resource_t *pres = 0;
     byte invert = 0;
     bool in_line = false;
+    gs_show_enum *show_enum = (gs_show_enum *)pdev->pte;
+    int x_offset, y_offset;
+    double width;
 
     /* Update clipping. */
     if (pdf_must_put_clip_path(pdev, pcpath)) {
@@ -148,33 +154,39 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	if (one == gx_no_color_index)
 	    return 0;
 	/* If a mask has an id, assume it's a character. */
-	if (id != gx_no_bitmap_id && sourcex == 0) {
+	if (id != gx_no_bitmap_id && sourcex == 0 && show_enum) {
+	    pdf_char_proc_t *pcp;
+
+	    if (show_enum->use_wxy_float)
+	        pdev->char_width.x = show_enum->wxy_float.x;
+	    else
+	        pdev->char_width.x = fixed2float(show_enum->wxy.x);
 	    pres = pdf_find_resource_by_gs_id(pdev, resourceCharProc, id);
 	    if (pres == 0) {	/* Define the character in an embedded font. */
-		pdf_char_proc_t *pcp;
-		double x_offset;
-		int y_offset;
-
 		gs_image_t_init_mask(&image, false);
 		invert = 0xff;
+		x_offset = x - (int)show_enum->pis->current_point.x;
+		y_offset = y - (int)show_enum->pis->current_point.y;
+		x -= x_offset;
+		y -= y_offset;
+		y -= h;
 		pdf_make_bitmap_image(&image, x, y, w, h);
-		y_offset = pdf_char_image_y_offset(pdev, x, y, h);
 		/*
 		 * The Y axis of the text matrix is inverted,
 		 * so we need to negate the Y offset appropriately.
 		 */
-		code = pdf_begin_char_proc(pdev, w, h, 0, y_offset, id,
+		code = pdf_begin_char_proc(pdev, w, h, 0, y_offset, x_offset, id,
 					   &pcp, &ipos);
 		if (code < 0)
 		    return code;
 		y_offset = -y_offset;
-		x_offset = psdf_round(pdev->char_width.x, 100, 10); /* See 
+		width = psdf_round(pdev->char_width.x, 100, 10); /* See 
 			pdf_write_Widths about rounding. We need to provide 
 			a compatible data for Tj. */
-		pprintg1(pdev->strm, "%g ", x_offset);
-		pprintd3(pdev->strm, "0 0 %d %d %d d1\n", y_offset, w, h + y_offset);
-		pprintd3(pdev->strm, "%d 0 0 %d 0 %d cm\n", w, h,
-			 y_offset);
+		pprintg1(pdev->strm, "%g ", width);
+		pprintd4(pdev->strm, "0 %d %d %d %d d1\n",  x_offset, -h + y_offset, w + x_offset, y_offset);
+		pprintd4(pdev->strm, "%d 0 0 %d %d %d cm\n", w, h, x_offset,
+			 -h + y_offset);
 		pdf_image_writer_init(&writer);
 		code = pdf_begin_write_image(pdev, &writer, gs_no_id, w, h, NULL, true);
 		if (code < 0)
@@ -185,7 +197,11 @@ pdf_copy_mono(gx_device_pdf *pdev,
 		/* We're under pdf_text_process. It set a high level color. */
 	    } else
 		set_image_color(pdev, one);
-	    pdf_make_bitmap_matrix(&image.ImageMatrix, x, y, w, h, h);
+	    pcp = (pdf_char_proc_t *) pres;
+	    x -= pdf_charproc_x_offset(pcp);
+	    y -= pdf_charproc_y_offset(pcp);
+	    y -= h;
+	    pdf_make_bitmap_image(&image, x, y, w, h);
 	    goto rx;
 	}
 	set_image_color(pdev, one);
@@ -221,7 +237,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	c[1] = psdf_adjust_color_index((gx_device_vector *)pdev, one);
 	pcs = gs_cspace_alloc(pdev->memory, &gs_color_space_type_Indexed);
 	if (pcs == NULL) {
-	    rc_decrement(pcs_base, "pdf_copy_mono");
+	    rc_decrement_cs(pcs_base, "pdf_copy_mono");
 	    return_error(gs_error_VMerror);
 	}
 	pcs->base_space = pcs_base;
@@ -260,8 +276,8 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	 * We don't have to worry about color space scaling: the color
 	 * space is always a Device space.
 	 */
-	code = pdf_color_space(pdev, &cs_value, NULL, pcs,
-			       &writer.pin->color_spaces, in_line);
+	code = pdf_color_space_named(pdev, &cs_value, NULL, pcs,
+			       &writer.pin->color_spaces, in_line, NULL, 0);
 	if (code < 0)
 	    return code;
 	pcsvalue = &cs_value;
@@ -410,8 +426,8 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     pdf_image_writer_init(piw);
     pdev->ParamCompatibilityLevel = pdev->CompatibilityLevel;
     if ((code = pdf_begin_write_image(pdev, piw, id, w, h, NULL, in_line)) < 0 ||
-	(code = pdf_color_space(pdev, &cs_value, NULL, pcs,
-				&piw->pin->color_spaces, in_line)) < 0 ||
+	(code = pdf_color_space_named(pdev, &cs_value, NULL, pcs,
+				&piw->pin->color_spaces, in_line, NULL, 0)) < 0 ||
 	(for_pattern < 2 || nbytes < 512000 ?
 	    (code = psdf_setup_lossless_filters((gx_device_psdf *) pdev,
 			&piw->binary[0], (gs_pixel_image_t *)pim, false)) :
@@ -470,7 +486,7 @@ gdev_pdf_fill_mask(gx_device * dev,
 
     if (width <= 0 || height <= 0)
 	return 0;
-    if (depth > 1 || (!gx_dc_is_pure(pdcolor) != 0 && pdcolor->type != &gx_dc_pattern))
+    if (depth > 1 || (!gx_dc_is_pure(pdcolor) != 0 && !(gx_dc_is_pattern1_color(pdcolor))))
 	return gx_default_fill_mask(dev, data, data_x, raster, id,
 				    x, y, width, height, pdcolor, depth, lop,
 				    pcpath);
@@ -573,7 +589,7 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 		pprints1(s, "%s\nendstream\n", buf);
 	    else
 		pprints1(s, "%sendstream\n", buf);
-	    pdf_end_resource(pdev);
+	    pdf_end_resource(pdev, resourcePattern);
 	} else {
 	    length_id = pdf_obj_ref(pdev);
 	    pprintld1(s, "%ld 0 R>>stream\n", length_id);
@@ -590,10 +606,10 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 	    }
 	    end = pdf_stell(pdev);
 	    stream_puts(s, "\nendstream\n");
-	    pdf_end_resource(pdev);
-	    pdf_open_separate(pdev, length_id);
+	    pdf_end_resource(pdev, resourcePattern);
+	    pdf_open_separate(pdev, length_id, resourceNone);
 	    pprintld1(pdev->strm, "%ld\n", end - start);
-	    pdf_end_separate(pdev);
+	    pdf_end_separate(pdev, resourceNone);
 	}
 	pres->object->written = true; /* don't write at end of page */
     }
