@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfi.c 11908 2010-11-23 08:46:10Z ken $ */
+/* $Id: gdevpdfi.c 12169 2011-02-18 11:25:24Z ken $ */
 /* Image handling for PDF-writing driver */
 #include "memory_.h"
 #include "math_.h"
@@ -1384,9 +1384,10 @@ pdf_resource_t *pdf_substitute_pattern(pdf_resource_t *pres)
 static int 
 check_unsubstituted2(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
 {
-    pdf_pattern_t *ppat = (pdf_pattern_t *)pres0;
+    pdf_pattern_t *ppat0 = (pdf_pattern_t *)pres0;
+    pdf_pattern_t *ppat1 = (pdf_pattern_t *)pres1;
 
-    return ppat->substitute == NULL;
+    return (ppat0->substitute == NULL && ppat1->substitute == NULL);
 }
 
 static int 
