@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gxblend1.c 10359 2009-11-19 20:17:02Z ray $ */
+/* $Id: gxblend1.c 11889 2010-11-05 22:12:05Z mvrhel $ */
 /* PDF 1.4 blending functions */
 
 #include "memory_.h"
@@ -145,7 +145,6 @@ extern unsigned int global_index;
 void
 pdf14_preserve_backdrop(pdf14_buf *buf, pdf14_buf *tos, bool has_shape)
 {
-
     /* make copy of backdrop for compositing */
     int x0 = max(buf->rect.p.x, tos->rect.p.x);
     int x1 = min(buf->rect.q.x, tos->rect.q.x);
@@ -158,47 +157,40 @@ pdf14_preserve_backdrop(pdf14_buf *buf, pdf14_buf *tos, bool has_shape)
 	byte *tos_plane = tos->data + x0 - tos->rect.p.x + (y0 - tos->rect.p.y) * tos->rowstride;
 	int i;
 	/*int n_chan_copy = buf->n_chan + (tos->has_shape ? 1 : 0);*/
-	int n_chan_copy = tos->n_chan + (tos->has_shape ? 1 : 0);
+        int n_chan_copy = tos->n_chan + (tos->has_shape ? 1 : 0) + (tos->has_tags ? 1 : 0);
 
 	for (i = 0; i < n_chan_copy; i++) {
-		byte *buf_ptr = buf_plane;
-		byte *tos_ptr = tos_plane;
-		int y;
+	    byte *buf_ptr = buf_plane;
+	    byte *tos_ptr = tos_plane;
+	    int y;
 
-		for (y = y0; y < y1; ++y) {
-			memcpy (buf_ptr, tos_ptr, width); 
-			buf_ptr += buf->rowstride;
-			tos_ptr += tos->rowstride;
-		}
-		buf_plane += buf->planestride;
-		tos_plane += tos->planestride;
+	    for (y = y0; y < y1; ++y) {
+		    memcpy (buf_ptr, tos_ptr, width); 
+		    buf_ptr += buf->rowstride;
+		    tos_ptr += tos->rowstride;
 	    }
-	if (has_shape && !tos->has_shape)
-	    memset (buf_plane, 0, buf->planestride);
+	    buf_plane += buf->planestride;
+	    tos_plane += tos->planestride;
+	}
+        if (has_shape && !tos->has_shape) {
+            if (tos->has_tags) {
+                buf_plane -= buf->planestride;
+            }
+            memset (buf_plane, 0, buf->planestride);
+        }
     }
-
-
 #if RAW_DUMP
-
     if (x0 < x1 && y0 < y1) {
-        
         byte *buf_plane = buf->data + x0 - buf->rect.p.x + 
             (y0 - buf->rect.p.y) * buf->rowstride;
-
         dump_raw_buffer(y1-y0, x1 - x0, buf->n_planes,
                     buf->planestride, buf->rowstride, 
                     "BackDropInit",buf_plane);
-
         global_index++;
-
     }
-
 #endif
-
 }
 
-
-
 void
 pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf, 
 	      int x0, int x1, int y0, int y1, int n_chan, bool additive,
@@ -228,8 +220,12 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
     int tos_shape_offset = n_chan * tos_planestride;
     int tos_alpha_g_offset = tos_shape_offset +
     (tos->has_shape ? tos_planestride : 0);
+    bool tos_has_tag = tos->has_tags;
+    int tos_tag_offset = tos_planestride * (tos->n_planes - 1);
     int nos_shape_offset = n_chan * nos_planestride;
     bool nos_has_shape = nos->has_shape;
+    bool nos_has_tag = nos->has_tags;
+    int nos_tag_offset = nos_planestride * (nos->n_planes - 1);
     byte *mask_tr_fn = NULL; /* Quiet compiler. */
 #if RAW_DUMP
     byte *composed_ptr = NULL;
@@ -237,13 +233,14 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
 
     rect_merge(nos->bbox, tos->bbox);
 
-    if_debug6('v', "pdf14_pop_transparency_group y0 = %d, y1 = %d, w = %d, alpha = %d, shape = %d, bm = %d\n",
+    if_debug6('v', "pdf14_pop_transparency_group y0 = %d, y1 = %d, w = %d, alpha = %d, shape = %d, tag =  bm = %d\n",
 			y0, y1, width, alpha, shape, blend_mode);
     if (nos->has_alpha_g)
 	nos_alpha_g_ptr = nos_ptr + n_chan * nos_planestride;
     else
 	nos_alpha_g_ptr = NULL;
 
+
     if (maskbuf != NULL) { 
        
         mask_tr_fn = maskbuf->transfer_fn;
@@ -330,31 +327,48 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
 	    if (nos_knockout) {
 		byte *nos_shape_ptr = nos_has_shape ?
 		    &nos_ptr[x + nos_shape_offset] : NULL;
+                byte *nos_tag_ptr = nos_has_tag ?
+                    &nos_ptr[x + nos_tag_offset] : NULL;
 		byte tos_shape = tos_ptr[x + tos_shape_offset];
-
+		byte tos_tag = tos_ptr[x + tos_tag_offset];
 		art_pdf_composite_knockout_isolated_8(nos_pixel,
 						    nos_shape_ptr,
+                                                    nos_tag_ptr,
 						    tos_pixel,
 						    n_chan - 1,
 						    tos_shape,
+                                                    tos_tag,
 						    pix_alpha, shape);
-	    } else if (tos_isolated) {
-		art_pdf_composite_group_8(nos_pixel, nos_alpha_g_ptr,
-				    tos_pixel, n_chan - 1,
-				    pix_alpha, blend_mode, pblend_procs);
-	    } else {
-		byte tos_alpha_g = tos_ptr[x + tos_alpha_g_offset];
-		art_pdf_recomposite_group_8(nos_pixel, nos_alpha_g_ptr,
-				    tos_pixel, tos_alpha_g, n_chan - 1,
-				    pix_alpha, blend_mode, pblend_procs);
-	    }
+            } else {
+                if (tos_isolated) {
+		    art_pdf_composite_group_8(nos_pixel, nos_alpha_g_ptr,
+				        tos_pixel, n_chan - 1,
+				        pix_alpha, blend_mode, pblend_procs);
+	        } else {
+		    byte tos_alpha_g = tos_ptr[x + tos_alpha_g_offset];
+		    art_pdf_recomposite_group_8(nos_pixel, nos_alpha_g_ptr,
+				        tos_pixel, tos_alpha_g, n_chan - 1,
+				        pix_alpha, blend_mode, pblend_procs);
+	        }
+                if (tos_has_tag) {
+                    if (pix_alpha == 255) {
+                        nos_ptr[x + nos_tag_offset] = tos_ptr[x + tos_tag_offset];     
+                    } else if (pix_alpha != 0 && tos_ptr[x + tos_tag_offset] != 
+                               GS_UNTOUCHED_TAG) {  
+                        nos_ptr[x + nos_tag_offset] = 
+                            (nos_ptr[x + nos_tag_offset] | 
+                            tos_ptr[x + tos_tag_offset]) &
+                            ~GS_UNTOUCHED_TAG;     
+                    }
+                }
+            }
 	    if (nos_has_shape) {
 		nos_ptr[x + nos_shape_offset] =
 		    art_pdf_union_mul_8 (nos_ptr[x + nos_shape_offset],
 					    tos_ptr[x + tos_shape_offset],
 					    shape);
-	    }
-        
+
+            }         
 	    /* Complement the results for subtractive color spaces */
 	    if (additive) {
 		for (i = 0; i < n_chan; ++i) {
@@ -393,7 +407,7 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
 #if RAW_DUMP
 
 	/* The group alpha should disappear */
-    dump_raw_buffer(y1-y0, width, tos->n_planes - tos->has_alpha_g,
+    dump_raw_buffer(y1-y0, width, tos->n_planes - tos->has_alpha_g - tos->has_shape,
                 nos_planestride, nos->rowstride, 
                 "Composed",composed_ptr);
 
@@ -404,42 +418,6 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
 
 }
 
-/*
- * Encode a list of smask colorant values into a gx_color_index_value.
- * This has its own encoder as it may have a different number of colorants
- * compared to the actual device.
- */
-gx_color_index
-pdf14_encode_smask_color(gx_device *dev, const gx_color_value	colors[],int ncomp)
-{
-    int drop = sizeof(gx_color_value) * 8 - 8;
-    gx_color_index color = 0;
-    int i;
-
-    for (i = 0; i < ncomp; i++) {
-	color <<= 8;
-	color |= (colors[i] >> drop);
-    }
-    return (color == gx_no_color_index ? color ^ 1 : color);
-}
-
-/*
- * Decode a gx_color_index value back to a list of colorant values.
-  * This has its own decoder as it may have a different number of colorants
- * compared to the actual device.*/
-int
-pdf14_decode_smask_color(gx_device * dev, gx_color_index color, gx_color_value * out, int ncomp)
-{
-    int i;
-
-    for (i = 0; i < ncomp; i++) {
-	out[ncomp - i - 1] = (gx_color_value) ((color & 0xff) * 0x101);
-	color >>= 8;
-    }
-    return 0;
-}
-
-
 /*
  * Encode a list of colorant values into a gx_color_index_value.
  */
@@ -458,6 +436,28 @@ pdf14_encode_color(gx_device *dev, const gx_color_value	colors[])
     return (color == gx_no_color_index ? color ^ 1 : color);
 }
 
+/*
+ * Encode a list of colorant values into a gx_color_index_value.
+   Stick the tag information at the end.  
+ */
+gx_color_index
+pdf14_encode_color_tag(gx_device *dev, const gx_color_value colors[])
+{
+    int drop = sizeof(gx_color_value) * 8 - 8;
+    gx_color_index color;
+    int i;
+    int ncomp = dev->color_info.num_components;
+    int tag_value;
+
+    /* Add in the tag information */
+    color = gs_current_object_tag(dev->memory);
+    for (i = 0; i < ncomp; i++) {
+	color <<= 8;
+	color |= (colors[i] >> drop);
+    }
+    return (color == gx_no_color_index ? color ^ 1 : color);
+}
+
 /*
  * Decode a gx_color_index value back to a list of colorant values.
  */
@@ -490,8 +490,9 @@ pdf14_compressed_encode_color(gx_device *dev, const gx_color_value colors[])
     if (pdevn_params == NULL) {
 #ifdef DEBUG
 	if (strncmp(dev->dname, "pdf14", 5))
-	    eprintf1("pdf14_compressed_encode_color devn_params not from pdf14 device, device = '%s'\n",
-			dev->dname);
+            emprintf1(dev->memory,
+                      "pdf14_compressed_encode_color devn_params not from pdf14 device, device = '%s'\n",
+                      dev->dname);
 #endif
     	pdevn_params = &(((pdf14_device *)dev)->devn_params);
     }
@@ -515,8 +516,9 @@ pdf14_compressed_decode_color(gx_device * dev, gx_color_index color,
     if (pdevn_params == NULL) {
 #ifdef DEBUG
 	if (strncmp(dev->dname, "pdf14", 5))
-	    eprintf1("pdf14_compressed_decode_color devn_params not from pdf14 device, device = '%s'\n",
-			dev->dname);
+            emprintf1(dev->memory,
+                      "pdf14_compressed_decode_color devn_params not from pdf14 device, device = '%s'\n",
+                      dev->dname);
 #endif
     	pdevn_params = &(((pdf14_device *)dev)->devn_params);
     }
@@ -556,7 +558,7 @@ pdf14_rgb_cs_to_cmyk_cm(gx_device * dev, const gs_imager_state *pis,
     int num_comp = dev->color_info.num_components;
 
     if (pis != 0)
-        color_rgb_to_cmyk(r, g, b, pis, out);
+        color_rgb_to_cmyk(r, g, b, pis, out, dev->memory);
     else {
         frac    c = frac_1 - r, m = frac_1 - g, y = frac_1 - b;
         frac    k = min(c, min(m, g));
@@ -722,6 +724,38 @@ gx_build_blended_image_row(byte *buf_ptr, int y, int planestride,
     }
 }
 
+void
+gx_blend_image_buffer(byte *buf_ptr, int width, int height, int rowstride, 
+                      int planestride, int num_comp, byte bg)
+{
+    int x, y;
+    int position;
+    byte comp, a;
+    int tmp, comp_num;
+
+    for (y = 0; y < height; y++) {
+        position = y * rowstride;
+        for (x = 0; x < width; x++) {
+	    /* composite RGBA (or CMYKA, etc.) pixel with over solid background */
+            a = buf_ptr[position + planestride * num_comp];
+	    if ((a + 1) & 0xfe) {
+	        a ^= 0xff;
+	        for (comp_num = 0; comp_num < num_comp; comp_num++) {
+		    comp  = buf_ptr[position + planestride * comp_num];
+		    tmp = ((bg - comp) * a) + 0x80;
+		    comp += (tmp + (tmp >> 8)) >> 8;
+		    buf_ptr[position + planestride * comp_num] = comp;
+	        }
+	    } else if (a == 0) {
+	        for (comp_num = 0; comp_num < num_comp; comp_num++) {
+		    buf_ptr[position + planestride * comp_num] = bg;
+	        }
+	    }
+            position+=1;
+        }
+    }
+}
+
 int
 gx_put_blended_image_cmykspot(gx_device *target, byte *buf_ptr, 
 		      int planestride, int rowstride,
