@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxccman.c 9466 2009-02-10 17:38:02Z marcos $ */
+/* $Id: gxccman.c 12119 2011-02-05 15:29:32Z chrisl $ */
 /* Character cache management routines for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -383,7 +383,7 @@ gx_lookup_xfont(const gs_state * pgs, cached_fm_pair * pair, int encoding_index)
     gs_font *font = pair->font;
     const gx_xfont_procs *procs = (*dev_proc(fdev, get_xfont_procs)) (fdev);
     gx_xfont *xf = 0;
-
+    
     /* We mustn't attempt to use xfonts for stroked characters, */
     /* because such characters go outside their bounding box. */
     if (procs != 0 && font->PaintType == 0) {
@@ -434,6 +434,11 @@ gx_lookup_xfont(const gs_state * pgs, cached_fm_pair * pair, int encoding_index)
 	    }
 	}
     }
+    
+    if (xf) {
+        emprintf(pgs->memory, "Warning: the Xfonts feature is deprecated and will be removed in a future release.\n");
+    }
+    
     pair->xfont = xf;
 }
 
@@ -608,6 +613,8 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
 	/* Pass the correct target, but decrement its refct afterwards. */
 	gs_make_mem_mono_device(pdev, pdev->memory, target);
 	rc_decrement_only(target, "gx_alloc_char_bits"); /* can't go to 0 */
+        /* Decrement the ICC profile also.  Same device is getting reinitialized */
+        if (target != NULL) rc_decrement(target->device_icc_profile,"gx_alloc_char_bits(icc profile)");
 	pdev->rc = rc;
 	pdev->retained = retained;
 	pdev->width = iwidth;
@@ -808,9 +815,11 @@ gx_add_char_bits(gs_font_dir * dir, cached_char * cc,
 	if_debug5('k', "[k]compressing %dx%d by %dx%d to depth=%d\n",
 		  cc->width, cc->height, 1 << log2_x, 1 << log2_y,
 		  depth);
+#ifdef DEBUG
 	if (gs_debug_c('K'))
 	    debug_dump_bitmap(bits, raster, cc->height,
 			      "[K]uncompressed bits");
+#endif
 	/* Truncate/round the bbox to a multiple of the scale. */
 	{
 	    int scale_x = 1 << log2_x;
