@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclpath.c 9352 2009-01-14 11:15:24Z leonardo $ */
+/*$Id: gxclpath.c 11865 2010-10-28 06:12:53Z mvrhel $ */
 /* Higher-level path operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -55,7 +55,7 @@ colored_halftone_colors_used(gx_device_clist_writer *cldev,
      * standard CMYK color mapping function.
      */
     if (dev_proc(cldev, map_cmyk_color) != cmyk_1bit_map_cmyk_color)
-	return ((gx_color_index)1 << cldev->color_info.depth) - 1;
+	return ((gx_color_index)1 << cldev->color_info.depth) - 1;  /* What about tranparency?  Need to check this */
     /*
      * Note that c_base[0], and the low-order bit of plane_mask,
      * correspond to cyan: this requires reversing the bit order of
@@ -186,7 +186,7 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	}
     }
 
-    while (left) {
+    do {
 	prefix_size = 2 + 1 + (offset > 0 ? enc_u_sizew(offset) : 0);
 	req_size = left + prefix_size + enc_u_sizew(left);
 	code = cmd_get_buffer_space(cldev, pcls, req_size);
@@ -223,10 +223,10 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	}
 	offset += portion_size;
 	left -= portion_size;
-    }
+    } while (left);
 
     /* should properly calculate colors_used, but for now just punt */
-    pcls->colors_used.or = ((gx_color_index)1 << cldev->color_info.depth) - 1;
+    pcls->colors_used.or = ((gx_color_index)1 << cldev->clist_color_info.depth) - 1;
 
     /* Here we can't know whether a pattern paints colors besides 
        black and white, so assume that it does.
@@ -273,7 +273,7 @@ cmd_drawing_colors_used(gx_device_clist_writer *cldev,
     else if (gx_dc_is_colored_halftone(pdcolor))
 	return colored_halftone_colors_used(cldev, pdcolor);
     else
-	return ((gx_color_index)1 << cldev->color_info.depth) - 1;
+	return ((gx_color_index)1 << cldev->clist_color_info.depth) - 1;
 }
 
 
@@ -429,8 +429,9 @@ cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	];
 	byte *bp = buf;
 
+        /* Here we assume that all the caps are the same as start_cap */
 	if (unknown & cap_join_known) {
-	    *bp++ = (cldev->imager_state.line_params.cap << 3) +
+	    *bp++ = (cldev->imager_state.line_params.start_cap << 3) +
 		cldev->imager_state.line_params.join;
 	}
 	if (unknown & cj_ac_sa_known) {
@@ -637,12 +638,20 @@ cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 		map_size = num_values;
 	    }
 	    code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_set_color_space,
-				  2 + cmd_sizew(hival) + map_size);
+				  2 + cmd_sizew(hival) + map_size + 
+                                  sizeof(clist_icc_color_t));
 	    if (code < 0)
 		return code;
-	    memcpy(cmd_put_w(hival, dp + 2), map_data, map_size);
+            /* Save the ICC information */
+            memcpy(dp + 2, &(cldev->color_space.icc_info),
+                   sizeof(clist_icc_color_t));
+	    memcpy(cmd_put_w(hival, dp + 2 + 
+                   sizeof(clist_icc_color_t)), map_data, map_size);
 	} else {
-	    code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_set_color_space, 2);
+	    code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_set_color_space, 
+                2 + sizeof(clist_icc_color_t));
+            memcpy(dp + 2, &(cldev->color_space.icc_info),
+                   sizeof(clist_icc_color_t));
 	    if (code < 0)
 		return code;
 	}
@@ -860,9 +869,10 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 			  pis->line_params.dot_length,
 			  pis->line_params.dot_length_absolute);
     }
-    if (state_neq(line_params.cap) || state_neq(line_params.join)) {
+    /* Here we assume that all the caps are the same as start_cap */
+    if (state_neq(line_params.start_cap) || state_neq(line_params.join)) {
 	unknown |= cap_join_known;
-	state_update(line_params.cap);
+	state_update(line_params.start_cap);
 	state_update(line_params.join);
     }
     cmd_check_fill_known(cdev, pis, params->flatness, &pis->fill_adjust,
