@@ -11,6 +11,8 @@ use Data::Dumper;
 
 my $verbose=1;
 
+my $skip="CLUSTER_UNTESTED";
+
 
 my $runningSemaphore="./clustermaster.pid";
 my $queue="./queue.lst";
@@ -276,22 +278,33 @@ sub checkProblem {
     die "svn info failed";
   }
   if ($currentRev1 != $newRev1 || $currentRev2 != $newRev2) {
-#   print "$currentRev1 $newRev1\n$currentRev2 $newRev2\n";
-    open(LOCK,">$lock") || die "can't write to $lock";
-    flock(LOCK,LOCK_EX);
     my $rev=$newRev1;
     $rev=$newRev2 if ($newRev2>$rev);
-    my $s="svn $rev";
-    my $t=`grep "$s" $queue`;
-    chomp $t;
-#   print "grep '$s' returned: $t\n";
-    if (length($t)==0) {
-      mydbg "  grep '$s' returns no match, adding to queue\n";
-      open(F,">>$queue");
-      print F "$s\n";
-      close(F);
+    my $logMessage;
+    $logMessage.=`svn log ghostpdl -r$rev`;
+    $logMessage.=`svn log ghostpdl/gs -r$rev`;
+    if ($logMessage =~ m/$skip/) {
+      mydbg "  $skip found in r$rev, skipping";
+      mydbg "  svn cleanup ghostpdl ; svn update ghostpdl -r$rev --ignore-externals\n";
+      mydbg "  svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$rev\n";
+      `svn cleanup ghostpdl ; svn update ghostpdl -r$rev --ignore-externals`;
+      `svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$rev`;
+    } else {
+#     print "$currentRev1 $newRev1\n$currentRev2 $newRev2\n";
+      open(LOCK,">$lock") || die "can't write to $lock";
+      flock(LOCK,LOCK_EX);
+      my $s="svn $rev";
+      my $t=`grep "$s" $queue`;
+      chomp $t;
+#     print "grep '$s' returned: $t\n";
+      if (length($t)==0) {
+        mydbg "  grep '$s' returns no match, adding to queue\n";
+        open(F,">>$queue");
+        print F "$s\n";
+        close(F);
+      }
+      close(LOCK);
     }
-    close(LOCK);
   }
 }
 
@@ -357,12 +370,12 @@ if (0) {
     if (open(F,"<$usersDir/$user/gs.run")) {
       close(F);
       unlink "$usersDir/$user/gs.run";
-      $product="$user gs pcl xps svg ls";
+      $product="gs pcl xps svg ls";
     }
     if (open(F,"<$usersDir/$user/ghostpdl.run")) {
       close(F);
       unlink "$usersDir/$user/ghostpdl.run";
-      $product="$user gs pcl xps svg ls";
+      $product="gs pcl xps svg ls";
     }
     if (open(F,"<$usersDir/$user/ghostpdl/cluster_command.run")) {
       $product=<F>;
@@ -383,12 +396,14 @@ if (0) {
     $options="" if (!$options);
     if ($product) {
       $product =~ s/ +$//;
-      mydbg "  user $product\n";
+      $product =~ s/^ +//;
+      $product =~ m/(.+?) (.+)/;
+      $user=$1;
+      $product=$2;
+      mydbg "  user $user product $product\n";
       open(LOCK,">$lock") || die "can't write to $lock";
       flock(LOCK,LOCK_EX);
       if ($product =~  m/abort$/) {
-        if ($product =~ m/^(.+) /) {
-          $user = $1;
 mylog "  abort for user $user\n";
           if (open(F,"<$queue")) {
             my $first=1;
@@ -412,26 +427,41 @@ mydbg "  setting 'abort.job'\n";
             }
             close(F);
           }
-        }
       } else {
-        my $s="user $product";
-        my $t=`grep "$s" $queue`;
-        chomp $t;
-        mydbg "  grep '$s' returned: $t\n";
-        if (length($t)==0) {
-          mydbg "  grep '$s' returns no match, adding to queue\n";
-          open(F,">>$queue");
-          print F "$s $options\n";
-          close(F);
-          mydbg "  running: find $usersDir/$user/ghostpdl -name \\*.sh | xargs \$HOME/bin/flip -u\n";
-          `find $usersDir/$user/ghostpdl -name \\*.sh | xargs \$HOME/bin/flip -u`;
-          mydbg "  running: find $usersDir/$user/ghostpdl -name instcopy | xargs \$HOME/bin/flip -u\n";
-          `find $usersDir/$user/ghostpdl -name instcopy | xargs \$HOME/bin/flip -u`;
-          mydbg "  running: chmod -R +xr $usersDir/$user/ghostpdl\n";
-          `chmod -R +xr $usersDir/$user/ghostpdl`;
+
+sub addToQueue($$) {
+  my $user=shift;
+  my $product=shift;
+  while($product =~ s/  / /g) {
+  }
+  $product =~ s/^ +//;
+  $product =~ s/ +$//;
+  my $s="user $user $product";
+  my $t=`grep "$s" $queue`;
+  chomp $t;
+  mydbg "  grep '$s' returned: $t\n";
+  if (length($t)==0) {
+    mydbg "  grep '$s' returns no match, adding to queue\n";
+    open(F,">>$queue");
+    print F "$s $options\n";
+    close(F);
+    mydbg "  running: find $usersDir/$user/ghostpdl -name \\*.sh | xargs \$HOME/bin/flip -u\n";
+    `find $usersDir/$user/ghostpdl -name \\*.sh | xargs \$HOME/bin/flip -u`;
+    mydbg "  running: find $usersDir/$user/ghostpdl -name instcopy | xargs \$HOME/bin/flip -u\n";
+    `find $usersDir/$user/ghostpdl -name instcopy | xargs \$HOME/bin/flip -u`;
+    mydbg "  running: chmod -R +xr $usersDir/$user/ghostpdl\n";
+    `chmod -R +xr $usersDir/$user/ghostpdl`;
+  }
+}
+        if ($product =~ m/bmpcmp/) {
+          $product =~ s/bmpcmp//;
+          addToQueue($user,$product) if (length($product));
+          addToQueue($user,'bmpcmp');
+        } else {
+          addToQueue($user,$product);
         }
-        close(LOCK);
       }
+      close(LOCK);
     }
   }
   closedir DIR;
@@ -601,10 +631,10 @@ if ($regression =~ m/svn (\d+)/) {
   chomp $currentRev1;
   chomp $currentRev2;
 
-  my $a=`svn update ghostpdl -r$rev --ignore-externals`;
-  my $b=`svn update ghostpdl/gs -r$rev`;
-  mydbg "svn update ghostpdl -r$rev --ignore-externals\n" if ($verbose);
-  mydbg "svn update ghostpdl/gs -r$rev\n" if ($verbose);
+  my $a=`svn cleanup ghostpdl ; svn update ghostpdl -r$rev --ignore-externals`;
+  my $b=`svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$rev`;
+  mydbg "svn cleanup ghostpdl ; svn update ghostpdl -r$rev --ignore-externals\n" if ($verbose);
+  mydbg "svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$rev\n" if ($verbose);
 
   $footer.="------------------------------------------------------------------------\n";
   $footer.=`svn log ghostpdl -r$rev | tail -n +2`;
@@ -657,10 +687,10 @@ if ($regression =~ m/svn (\d+)/) {
 
   if (length($product)) {
 # un-update the source so that if the regression fails were are back to the where we started
-    `svn update ghostpdl -r$currentRev1 --ignore-externals`;
-    `svn update ghostpdl/gs -r$currentRev2`;
-    mydbg "svn update ghostpdl -r$currentRev1 --ignore-externals\n";
-    mydbg "svn update ghostpdl/gs -r$currentRev2\n";
+    `svn cleanup ghostpdl ; svn update ghostpdl -r$currentRev1 --ignore-externals`;
+    `svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$currentRev2`;
+    mydbg "svn cleanup ghostpdl ; svn update ghostpdl -r$currentRev1 --ignore-externals\n";
+    mydbg "svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$currentRev2\n";
   } else {
     mydbg "no interesting files changed, skipping regression\n";
     $normalRegression=0;
@@ -734,6 +764,11 @@ if ($normalRegression==1 || $userRegression ne "" || $mupdfRegression==1 || $upd
 #   my $t=time-stat("$_")->mtime;
 #   print "$_ $t\n";
     $machines{$_}=1 if (stat("$_") && (time-stat("$_")->mtime)<$maxTouchTime);
+    # set the status of all machines to idle, so that old messages don't confuse the dashboard
+    s/.up/.status/;
+    open(F,">$_");
+    print F "idle\n";
+    close(F);
   }
   foreach (keys %machines) {
     delete $machines{$_};
@@ -1438,9 +1473,9 @@ close(F9);
 #   `mail marcos.woehrmann\@artifex.com -s \"\`cat revision.gs\`\" <email.txt`;
 
     mydbg "test complete, performing final svn update\n";
-    mydbg "svn update ghostpdl -r$rev --ignore-externals\nsvn update ghostpdl/gs -r$rev\n";
-    `svn update ghostpdl -r$rev --ignore-externals`;
-    `svn update ghostpdl/gs -r$rev`;
+    mydbg "svn cleanup ghostpdl ; svn update ghostpdl -r$rev --ignore-externals\nsvn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$rev\n";
+    `svn cleanup ghostpdl ; svn update ghostpdl -r$rev --ignore-externals`;
+    `svn cleanup ghostpdl/gs ; svn update ghostpdl/gs -r$rev`;
 
     `./cp.all.sh`;
 mydbg("calling cachearchive.pl");
@@ -1462,7 +1497,7 @@ mydbg("finished cachearchive.pl");
   } elsif ($updateBaseline) {
   } elsif ($bmpcmp) {
     if (exists $emails{$userName}) {
-      `cp bmpcmpFinished.txt bmpcmpResults.txt`;
+      `mv bmpcmpResults.txt bmpcmpResults.txt.old`;
       `echo >>bmpcmpResults.txt`;
       `echo http://www.ghostscript.com/~regression/$userName >>bmpcmpResults.txt`;
       `echo >>bmpcmpResults.txt`;
